@@ -26,6 +26,7 @@ sub init {
     my ($self, $class) = @_;
 
     eval "require $class";
+    die $@ if $@;
 
     foreach my $entry (@{ $self->{entries} }) {
         bless $entry, $class;
@@ -33,8 +34,6 @@ sub init {
     }
 }
 
-=head1 
-
 =head1 SEE ALSO
 
 =over 4
